{===============================================================================

  Ce composant permet d'afficher la version du programme dans lequel il se
  trouve. Le numro de version n'apparat qu' l'xcution.
  Merci  Nono40 (http://nono40.developpez.com) pour la fonction principale.

  cyril.michy@hotmail.com

===============================================================================}
unit Version;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Graphics, Forms;

type
  TVersion = class(TcustomControl)
  private
    FTextBefore,FVersion,FCaption: shortstring;
    FBorderStyle: TBorderStyle;
    FXOff, FYOff: Integer;
    FWithoutBuildVersion:boolean;
    FShadow:boolean;
    FShadowW:byte;
    FShadowColor:TColor;
    procedure SetBorderStyle(Value:TBorderStyle);
    procedure SetTextBefore(Value:Shortstring);
    function NoVersion: shortstring;
    procedure ChangeVersionResult(Value:boolean);
    procedure SetShadow(Value:boolean);
    procedure SetShadowWidth(Value:byte);
    procedure SetShadowColor(Value:TColor);
  protected
    procedure Paint; override;
  public
   constructor Create(AOwner:TComponent); override;
  published
    property BorderStyle:TBorderStyle read FBorderStyle write SetBorderStyle;
    property Color;
    property ExeVersion:shortstring read FVersion;
    property Font;
    property ShadowColor:TColor read FShadowColor write SetShadowColor;
    property ShadowWitdh: byte read FShadowW write SetShadowWidth default 2;
    property TextShadow:boolean read FShadow write SetShadow;
    property TextBefore:shortstring read FTextBefore write SetTextBefore;
    property WithoutBuildVersion:boolean read FWithoutBuildVersion write ChangeVersionResult;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('Util', [TVersion]);
end;

{ TVersion }

procedure TVersion.ChangeVersionResult(Value: boolean);
begin
  FWithoutBuildVersion:=Value;
  FVersion:=NoVersion;
  FCaption:=Concat(FTextBefore,FVersion);
  Invalidate;
end;

constructor TVersion.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);
  FTextBefore:='V ';
  FVersion:=NoVersion;
  FCaption:=Concat(FTextBefore,FVersion);
  FBorderStyle:=bsNone;
  FWithoutBuildVersion:=False;
  Width:=65;
  Height:=15;
  FShadow:=false;
  FShadowW:=2;
  FShadowColor:=clWhite;
end;

function TVersion.NoVersion: shortstring;
{ --- Code crit par Nono40 (http://nono40.developpez.com) --- }
{-}var
{-}  S, temp: string;
{-}  Size: DWord;
{-}  Buffer: PChar;
{-}  VersionPC: PChar;
{-}  VersionL: DWord;
{-}begin
{-}  Result:='0.0.0.0'; // Valeur par dfaut
{-}  Buffer:='';
{-}  S := Application.ExeName;
{-}
{-}  Size := GetFileVersionInfoSize(PChar(S), Size);
{-}  if Size>0 then
{-}    try
{-}      {--- Rservation en mmoire d'une zone de la taille voulue ---}
{-}      Buffer := AllocMem(Size);
{-}      {--- Copie dans le buffer des informations ---}
{-}      GetFileVersionInfo(PChar(S), 0, Size, Buffer);
{-}      {--- Recherche de l'information de version ---}
{-}      if VerQueryValue(Buffer, PChar('\StringFileInfo\040C04E4\FileVersion'),
{-}         Pointer(VersionPC), VersionL) then
{-}         Result:=VersionPC;
{-}    finally
{-}      FreeMem(Buffer, Size);
{-}    end;
{ --- Code crit par Nono40 (http://nono40.developpez.com) --- }

  { --- Suivant affichage demand --- }
  if FWithoutBuildVersion then begin
    temp:=Result;
    while pos('.',Result)<>0 do
      Result:=copy(Result,pos('.',Result)+1,length(Result)-pos('.',Result));
    Result:=copy(temp,0,length(temp)-length(Result)-1);
  end;
end;

procedure TVersion.Paint;
var
  tmpC:TColor;
begin
  inherited Paint;
  with Canvas do begin
    Font:=Self.Font;
    Brush.Color:=Self.Color;

    { --- Positionnement du texte --- }
    FXOff:=(Width-TextWidth(FCaption)) div 2;
    FYOff:=(Height-TextHeight(FCaption)) div 2;
    Width:=TextWidth(FCaption)+(2*FXOff);
    Height:=TextHeight(FCaption)+(2*FYOff);

    { --- Encadr --- }
    if (FBorderStyle<>bsNone) then
      Rectangle(ClientRect);

    { --- Ecriture du numro de version --- }
    if FShadow then begin
      Brush.Style:=bsClear;
      tmpC:=Font.Color;
      Font.Color:=FShadowColor;
      TextOut(FXOff+FShadowW,FYOff+FShadowW,FCaption);
      Font.Color:=tmpC;
    end;
    TextOut(FXOff,FYOff,FCaption);
  end;
end;

procedure TVersion.SetBorderStyle(Value: TBorderStyle);
begin
  FBorderStyle:=Value;
  Invalidate;
end;

procedure TVersion.SetShadow(Value: boolean);
begin
  FShadow:=Value;
  Invalidate;
end;

procedure TVersion.SetShadowColor(Value: TColor);
begin
  FShadowColor:=Value;
  Invalidate;
end;

procedure TVersion.SetShadowWidth(Value: byte);
begin
  if (Value<=10) and (Value>0) then
    FShadowW:=Value
  else
    FShadowW:=2;
  Invalidate;
end;

procedure TVersion.SetTextBefore(Value: Shortstring);
begin
  FTextBefore:=Value;
  FCaption:=Concat(FTextBefore,FVersion);
  Invalidate;
end;

end.
